# -*- coding: utf-8 -*-
import global_defines
from global_defines import Lambda
from global_defines import data
import character
import globaltext
import system
import music
import string
import comments
import timerepr
import object_events
import random
import objects
import zone_ambients
import spawnanimals
import helpfile
__doc__= """This module contains callback functions for system events.

Most of the functions here can be overloaded by the zone script. If you do that, it
is your responsibility to ensure the overloading function implements the necessary
behavior or calls the default handler.
"""

def on_map_loaded():
	"""Called when the player party enters a zone they haven't visited before.
Called when everything has been initialized, but before on_finally_initialized.
"""
	zone_ambients.init_ambient()
	music.on_map_loaded()

	# make sure every char has perception values set	
	calc_perception_values()

	# init/reinit rest module
	import rest
	system.notify_global_timer("cyclic_fatigue_check", rest.cyclic_fatigue_check_interval)
	system.notify_global_timer("poisoning", 1000*120)
	print "poison timer_mapload"
	rest_while_traveling()
	
	set_follower_char_states()
	
	# Spawnanimal Daten sammeln
	spawnanimals.set_on_map_loaded_data()

	# bug 13964: speed_factor might be inkorrekt after zone change
	for char in system.get_pcs():
		object_events.update_speed(char)
		
	# bug 15373: fallback; reset on zone change
	data("rest").raid = None
	data("rest").resting = False
	
	if not system.is_global_timer_active("bonus_malus_timer"):
		system.notify_global_timer("bonus_malus_timer", 15*1000, repeating=True)

def rest_while_traveling():
	"""Sets a new "last rest time" so chars aren't fatigued when they enter a zone.

Called by on_map_loaded & on_map_reentry.
"""
	time = timerepr.time
	if not hasattr(data("rest"), "last_rest_time"): data("rest").last_rest_time = time("0:0:0")
	if system.get_mission_time() < data("rest").last_rest_time:
		data("rest").last_rest_time = system.get_mission_time()
	time_since_last_rest = time("%d:%d:%d" % (random.randint(0,10), random.randint(0,59), random.randint(0,59)))
	if time_since_last_rest > system.get_mission_time(): time_since_last_rest = system.get_mission_time()
	data("rest").last_rest_time = system.get_mission_time() - time_since_last_rest
	
def on_map_exit(next_zone):
	"""Called when the player leaves a zone (usually via travel map).

Parameters:
        next_zone - The id of the zone that will be entered (e.g., "zone_5.zone_5").
"""
	system.backpack_clear()
	if hasattr(data("ambient"), "ambient_set"):
		if data("ambient").ambient_set:
			zone_ambients.clear_ambient()
	data("rest").last_mission_time = system.get_mission_time()
	if global_defines.LOCAL_USER == global_defines.USER_TORSTEN_VOLLRATH:
		import debug
		debug.render_debug('ALTER_EGO', 'delete')
	object_events.remove_bereft_attribute()
	comments.on_map_exit( next_zone )

def on_finally_initialized():
	"""Called when a zone has been loaded and been completely initialized.

Called immediately before the first game tick.
"""
	helpfile.itemers_check()
	music.on_finally_initialized()
	comments.on_finally_initialized()


def on_init():
	"""Called when a zone is initialized (i.e., the loading of a zone is started).
"""
	import rest
	rest.init_data()
	
def on_map_reentry():
	"""Called when the player party reenters a zone they have visited before.

Called when everything has been initialized, but before on_finally_initialized.
"""
	if hasattr(data("ambient"), "ambient_set"):
		if data("ambient").ambient_set:
			zone_ambients.change_ambient()
	music.on_map_reentry()
	# init/reinit rest module
	import rest
	rest.init_data()
	system.notify_global_timer("cyclic_fatigue_check", rest.cyclic_fatigue_check_interval)
	rest_while_traveling()

	if not hasattr(data("poison"), "poison_timer_run"):	
		data("poison").poison_timer_run =True
		print "poison timer_reentry"
		system.notify_global_timer("poisoning", 1000*120)

	
	set_follower_char_states()
	
	#Spawnanimals Wiederbeleben
	spawnanimals.check_spawnanimals_revival()

	# bug 13964: speed_factor might be inkorrekt after zone change
	for char in system.get_pcs():
		object_events.update_speed(char)

	# bug 15373: fallback; reset on zone change
	data("rest").raid = None
	data("rest").resting = False
	
	#  bug 15161: no slot limit for trader inventories
	for character_id in system.get_npcs():
		if (objects.has_attribute(character_id, "trader") and
			objects.get_attribute(character_id, "trader")):
			objects.set_attribute(character_id, "inventory_overstock_behaviour", "enlarge_inventory")

	if not system.is_global_timer_active("bonus_malus_timer"):
		system.notify_global_timer("bonus_malus_timer", 15*1000, repeating=True)

def on_post_load():
	"""Called after a savegame has been loaded.
"""
	if global_defines.LOCAL_USER == global_defines.USER_ALEXANDER_MISELER:
		import debug
		debug.cheat("region")
		debug.cheat("region")
		debug.cheat("tile")
		debug.cheat("tile")

	music.on_post_load()
	import rest
	rest.init_data()
	# hack, some players have savegames where they are stuck with data("rest").rest_after_raid==True
	if hasattr(data("rest"), "rest_after_raid") and data("rest").rest_after_raid:
		if not system.in_combat_mode():
			data("rest").rest_after_raid = False
	if not hasattr(data("poison"), "poison_timer_run"):	
		data("poison").poison_timer_run =True
		print "poison timer_postload"
		system.notify_global_timer("poisoning", 1000*120)
	
	set_follower_char_states()

	#replace old talents
	replace_talents = {	
		"arm_vehicles"	:["technic","stronger_armor_parts"],
		"road_battle"	:["driving","gentle_driving"],
		"tanks"			:["driving","repair_vehicles"],
		"helicopters"	:["driving","road_trip"],
		"persuade"		:["speech","entertainment"],
		"conversation"	:["speech","teach"],
	}

	#  bug 15161: no slot limit for trader inventories
	for character_id in system.get_npcs():
		if (objects.has_attribute(character_id, "trader") and
			objects.get_attribute(character_id, "trader")):
			objects.set_attribute(character_id, "inventory_overstock_behaviour", "enlarge_inventory")

	for character_id in system.get_pcs(include_unconscious=True):
		for talent in replace_talents:
			char_talents = global_defines.helper_get_talents( character_id , replace_talents[talent][0] )
			if talent in char_talents:
				index = char_talents.index(talent)
				if index >= 0:
					char_talents[index]=replace_talents[talent][1]
					print "talent replaced:",character_id,talent,replace_talents[talent][1]

		for act_skill in global_defines.all_talents.keys():
			object_events.check_talents(character_id,act_skill)
			
		#remove wrong foes setting for pcs
		if objects.has_attribute(character_id, "foes"):
			if "UISPIELER" in objects.get_attribute(character_id, "foes"):
				foes = objects.get_attribute(character_id, "foes")
				foes.remove("UISPIELER")
				objects.set_attribute(character_id, "foes", foes)
		
	if hasattr(data("ambient"), "ambient_set"):
		if data("ambient").ambient_set:
			zone_ambients.change_ambient()

	# remove weapon accustom bonus from old savegames
	for char in system.get_pcs():
		itemid = character.get_primary_weapon(char)
		itemtype = objects.get_attribute(itemid, "typeid")
		slot = objects.get_attribute(itemtype, "occupied_slots")[0]
		_type = objects.get_attribute(itemtype, "weapon_type")
		assert len(_type)
		type_list = ["schwer", "leicht", "sniper"]
		skill_list = ["heavy_weapons", "light_weapons", "sniper_weapons"]
		if _type in type_list:
			if objects.has_attribute(char, "weapon_accustom_skill_bonus" + slot):
				bonus = objects.get_attribute(char, "weapon_accustom_skill_bonus" + slot)
				skill = skill_list[type_list.index(_type)]
				objects.set_attribute(char, skill,  objects.get_attribute(char, skill) - bonus)
				# some kind of bonus must have been set; possible that it is zero
				# but the attribute has to be there
				assert objects.has_attribute(char, "weapon_accustom_skill_bonus" + slot)
				# remove attribute
				objects.set_attribute(char, "weapon_accustom_skill_bonus" + slot, None)

	# remove psychology bonus from old savegames
	if system.is_global_timer_active("psychology_check"):
		for char in system.get_pcs():
			object_events.remove_psychology_bonus(char)
	
	# remove increase moral & survive bonuses from old savegames
	remove_old_fatigue = False
	for char in system.get_pcs():
		print char
		if objects.has_attribute(char, "increase_moral_activated"):
			object_events.remove_increase_moral_bonus(char)

		object_events.remove_survive_bonus(char)

		if objects.has_attribute(char, "fatigue"):
			remove_old_fatigue = True
	
	# remove fatigue malus from old savegames
	if remove_old_fatigue:
		def calc_fatigue(att_penalty, skill_penalty, hp_factor):
			def sub_calc_fatigue(char, att_or_skill, penalty, fatigue_dict):
				changed = False
				dict_value = 0
				if att_or_skill in fatigue_dict:
					dict_value = fatigue_dict[att_or_skill]
					penalty -= dict_value
				value = objects.get_attribute(char, att_or_skill)
				import types
				if type(value) not in (types.IntType, types.LongType):
					exc = "illegal type %s in attribute/skill '%s' for char '%s' (value: <%s>)" % (
							str(type(value)), att_or_skill, char, repr(value))
					raise TypeError, exc 
				diff = min(penalty, value-1)
				if diff != 0:
					changed = True
					objects.set_attribute(char, att_or_skill, value-diff)
					fatigue_dict[att_or_skill] = dict_value + diff
				return changed
			
			changed = False
			import skill_tables
			import math
			for char in system.get_pcs():
				if objects.has_attribute(char, "fatigue"):
					fatigue = objects.get_attribute(char, "fatigue")
				else:
					fatigue = {}
				for att in skill_tables.attributes:
					change = sub_calc_fatigue(char, att, att_penalty, fatigue)
					changed = changed or change 
				for skill in skill_tables.skills:
					change = sub_calc_fatigue(char, skill, skill_penalty, fatigue)
					changed = changed or change 
	
				objects.set_attribute(char, "fatigue", fatigue)
	
				act_hp = objects.get_attribute(char, "hitpoints")
				max_hp = objects.get_attribute(char, "maxhitpoints")
				hp = min(act_hp, math.ceil(max_hp * hp_factor))
				objects.set_attribute(char, "hitpoints", hp)
			return changed
		att_penalty = 0
		skill_penalty = 0
		hp_factor = 1.0
		calc_fatigue(att_penalty, skill_penalty, hp_factor)

		# reset to make sure fatigue is calculated soon
		data("rest").state_nr = -1
		data("rest").state_change_chars = []

	# convert old kill stats - 13.01.2006
	from global_defines import kill_type_list
	for char in system.get_pcs():
		weapon_id_history={}
		weapon_type_history={}
		if objects.has_attribute(char, "weapon_type_history"):
			weapon_type_history = objects.get_attribute(char, "weapon_type_history")
		if objects.has_attribute(char, "weapon_id_history"):
			weapon_id_history = objects.get_attribute(char, "weapon_id_history")
		
		for weapon_type in weapon_type_history:
			kill_list = weapon_type_history[weapon_type]
			if len(kill_list) != len(kill_type_list):
				new_kill_list = [0] * len(kill_type_list)
				new_kill_list[kill_type_list.index("UNKNOWN_NPCS")] = kill_list[0]
				new_kill_list[kill_type_list.index("UNKNOWN_ANIMALS")] = kill_list[1]
				kill_list = new_kill_list
			weapon_type_history[weapon_type] = kill_list
		objects.set_attribute(char, "weapon_type_history", weapon_type_history)

		for weapon_id in weapon_id_history:
			kill_list = weapon_id_history[weapon_id]
			if len(kill_list) != len(kill_type_list):
				new_kill_list = [0] * len(kill_type_list)
				new_kill_list[kill_type_list.index("UNKNOWN_NPCS")] = kill_list[0]
				new_kill_list[kill_type_list.index("UNKNOWN_ANIMALS")] = kill_list[1]
				kill_list = new_kill_list
			weapon_id_history[weapon_id] = kill_list
		objects.set_attribute(char, "weapon_id_history", weapon_id_history)
	
		object_events.helper_add_weapon_accustomisation(char, itemtype, itemid)

	if not system.is_global_timer_active("bonus_malus_timer"):
		system.notify_global_timer("bonus_malus_timer", 15*1000, repeating=True)
	
	# hotfix for bug 16016: frozen timers in savegame
	if system.timer_frozen():
		if not hasattr(data("rest"), "resting") or not data("rest").resting:
			system.unfreeze_timer()

def on_end_audio_stream(player_id):
	"""Called when an audio stream ends.

This is mostly used to start new ambient/background/travelmusic.

Parameters:
        player_id - Id of the ending audio stream.
"""
	music.on_end_audio_stream(player_id)

def on_end_video_stream(player_id):
	"""Called when a video stream ends.

Parameters:
        player_id - Id of the ending video stream.
"""
	pass

def on_begin_combat():
	"""Called when the player chars attack or are attacked.
"""
	music.on_begin_combat()
	comments.on_begin_combat()

def on_end_combat():
	"""Called when the player chars are no longer fighting.
"""
	music.on_end_combat()
	comments.on_end_combat()
	
	print "*"*80
	print "* on_end_combat"
	print "*"*80

	continue_rest()

def continue_rest():
	"""Called by on_end_combat to continue the rest after a rest raid.
"""
	# check if this was a rest raid, continue rest if it was
	data = global_defines.data("rest")
	if hasattr(data, "rest_after_raid") and data.rest_after_raid:
		if hasattr(data, "can_continue_rest") and data.can_continue_rest:
			if not system.in_combat_mode():
				data.can_continue_rest = False
				if hasattr(data,"pcs"):
					act_pcs = system.get_pcs()
					if len(act_pcs)<len(data.pcs):
						system.do_request("REST_COMBAT_END_PC_VICTIMS")
					else:
						system.do_request("REST_COMBAT_END_NO_PC_VICTIMS")
				#ronny: wird jetzt durch requester getstartet
				#rest.start_rest()

def on_open_travel_map():
	"""Called when the travel map is opened.
"""
	music.on_open_travel_map()

def on_close_travel_map():
	"""Called when the travel map is closed.
"""
	music.on_close_travel_map()

def on_begin_cutscene():
	"""Called when a cutscene is started.
""" 
	mobs = system.get_pcs() + system.get_npcs() + system.get_animals()
	
	handle_vehicles_in_cutscene_on_begin()
	
	for mob in mobs:
		save_foes(mob)
		save_speedfactor(mob)
		
	# make sure AE is available
	char = "ALTER_EGO"
	if objects.is_unconscious(char):
		if character.is_carried(char):
			for carrier in system.get_pcs():
				if character.is_carrying_char(carrier)\
				and objects.has_attribute(carrier, "carried_char")\
				and objects.get_attribute(carrier, "carried_char") == char:
					character.dissolve_carrying(carrier)
		else:
			object_events.on_resuscitate(None, char, False)
		
	# makes player chars stand up
	#import dynamic_camera
	for pc in system.get_pcs():
		objects.cancel_commandos(pc)
		character.change_body_mode(pc, "STAND")
	
	#deactivate visibility fading to make chars + animals visible in cutscene
	for _id in (system.get_npcs() + system.get_animals()):
		objects.set_attribute(_id, "visibility", global_defines.UIVIS_VISIBLE)
		objects.set_attribute(_id, "visibility_fade", False)

	for _id in system.get_vehicles():
		objects.set_attribute(_id, "visibility", global_defines.UIVIS_VISIBLE)
		objects.set_attribute(_id, "visibility_fade", False)

def on_end_cutscene():
	"""Called when a cutscene ends.
""" 
	mobs = system.get_pcs() + system.get_npcs() + system.get_animals()
	
	for mob in mobs:
		restore_foes(mob)
		restore_speedfactor(mob)
	
	handle_vehicles_in_cutscenes_on_end()

	#activate visibilty fading
	for _id in (system.get_npcs() + system.get_animals()):
		objects.set_attribute(_id,"visibility_fade", True)

	for _id in system.get_vehicles():
		objects.set_attribute(_id,"visibility_fade", True)

## foe handling
def save_foes(char):
	"""Saves a chars foes attribute and clears it.

Parameters:
        char - Id of the char.
"""
	if objects.has_attribute(char, "foes"):
		f = objects.get_attribute(char, "foes")
		objects.set_attribute(char, "cutscene_temp_foes", f)
		objects.set_attribute(char, "foes", None)

def restore_foes(char):
	"""Restores a chars foes attribute from saved value.

Parameters:
        char - Id of the char.
"""
	if objects.has_attribute(char, "cutscene_temp_foes"):
		cstf = objects.get_attribute(char, "cutscene_temp_foes")
		objects.set_attribute(char, "foes", cstf)
		objects.set_attribute(char, "cutscene_temp_foes", None)

## speedfactor handling
def save_speedfactor(char):
	"""Saves a chars speedfactor if available, and sets speed factor to 1.0.

Parameters:
        char - Id of the char.
"""
	if objects.has_attribute(char, "speed_factor"):
		sf = objects.get_attribute(char, "speed_factor")
		objects.set_attribute(char, "saved_speed_factor", sf)
		objects.set_attribute(char, "speed_factor", 1.0)

def restore_speedfactor(char):
	"""Restores the chars original speedfactor.

Parameters:
        char - Id of the char.
"""
	if objects.has_attribute(char, "saved_speed_factor"):
		ssf = objects.get_attribute(char, "saved_speed_factor")
		objects.set_attribute(char, "speed_factor", ssf)
		objects.set_attribute(char, "saved_speed_factor", None)


def leave_zone():
	"""Shows the travel map.
"""
	system.show_zone_map()

def on_region_click( regionname, x, y ):
	"""Called when the player clicks on a region with the mouse.

Also see system.add_region.

Parameters:
        regionname - Id of the region (as set with add_region).
        x,y        - Coordinates player clicked on.
"""
	if	hasattr(data(), "registered_exit_zones") and \
		regionname in data().registered_exit_zones:

		if not can_leave_zone():
			if is_pc_carrying():
				system.do_request("CANT_LEAVE_ZONE_CARRY_PERSON")
			elif (hasattr(data("rest"), "raid") and data("rest").raid):
				system.do_request("CANT_LEAVE_ZONE_COMBAT")
			elif system.in_combat_mode():
				system.do_request("CANT_LEAVE_ZONE_COMBAT")
			else:
				system.do_request("CANT_LEAVE_ZONE_QUEST")
			return False

		if (hasattr(system.get_act_mission(),"leave_zone")):
			system.get_act_mission().leave_zone()
		else:
			leave_zone()
	else:
		return False

def can_leave_zone():
	"""Returns True if the player chars can leave the zone.
"""
	can_leave = True

	if (hasattr(system.get_act_mission(),"can_leave_zone")):
		can_leave = system.get_act_mission().can_leave_zone()

	# if one char is carrying another than the party cannnot leave the zone
	if is_pc_carrying():
		can_leave = False

	if system.in_combat_mode():
		can_leave = False

	if is_pc_in_car_without_fuel():
		can_leave = False

	return can_leave

def can_leave_zone_context_menu():
	"""Returns True if the "leave zone" context menu entry should be active.
"""
	var_can_leave_zone_context_menu = True
	if (hasattr(system.get_act_mission(),"can_leave_zone_context_menu") and
		callable(system.get_act_mission().can_leave_zone_context_menu)):
		var_can_leave_zone_context_menu = system.get_act_mission().can_leave_zone_context_menu()

	return (var_can_leave_zone_context_menu and can_leave_zone())

def is_pc_in_car_without_fuel():
	"""Returns True if one of the PCs is in a vehicle that has no fuel
"""
	for _id in system.get_vehicles():
		import vehicle
		crew = vehicle.get_crew(_id)
		for crew_member in crew:
			if crew_member in system.get_pcs():
				if objects.has_attribute(_id, "current_fuel_quantity"):
					if objects.get_attribute(_id, "current_fuel_quantity") == 0: 
						return True
	return False

def is_pc_carrying():
	"""Returns True if one of the PCs carrys another character
"""
	for pc in system.get_pcs():
		if objects.has_attribute( pc, "is_carrying" ):
			is_carrying = objects.get_attribute( pc, "is_carrying" )
			if is_carrying:
				return True
	return False

def on_leave_zone_context_menu():
	"""Called when the "leave zone" context menu entry is used.
"""
	if not can_leave_zone_context_menu():
		if is_pc_carrying():
			system.do_request("CANT_LEAVE_ZONE_CARRY_PERSON")
		elif is_pc_in_car_without_fuel():
			system.do_request("CANT_LEAVE_ZONE_CAR_WITHOUT_FUEL")
		elif system.in_combat_mode():
			system.do_request("CANT_LEAVE_ZONE_COMBAT")
		elif (hasattr(data("rest"), "raid") and data("rest").raid):
			system.do_request("CANT_LEAVE_ZONE_COMBAT")
		else:
			system.do_request("CANT_LEAVE_ZONE_QUEST")
	else:
		if (hasattr(system.get_act_mission(),"leave_zone")):
			system.get_act_mission().leave_zone()
		else:
			leave_zone()

def on_timer(id):
	"""Called when a global timer fires.

Timer can be started via system.notify_global_timer.

Parameters:
        id - Id of the timer event.
"""
	if id=="poisoning":
		system.notify_global_timer("poisoning", 1000*120)
		if not( hasattr(data("rest"), "resting") and data("rest").resting) :
			for character_id in  system.get_pcs(include_unconscious = True):
				if(objects.has_attribute(character_id,"poison_ticks")):
					if objects.get_attribute(character_id,"poison_ticks") > 0: #pc is poisoned
						objects.set_attribute(character_id,"poison_ticks",objects.get_attribute(character_id,"poison_ticks")-2)
						objects.damage( character_id, 2 )

		# heist failure tests, misuse the poison timer for it
		# if the last heist was more than 10 min ago then delete the attribute
		for member in system.get_pcs():
			if objects.has_attribute( member, "failed_heist" ):
				print  objects.get_attribute( member, "failed_heist" )
				if objects.get_attribute( member, "failed_heist" )>0:
					objects.set_attribute(member, "failed_heist",  objects.get_attribute( member, "failed_heist" )-(120*1000))
					if objects.get_attribute( member, "failed_heist" )<=0:
						system.add_text(globaltext.NPC_BECOME_FRIENDLY % {"char": objects.get_attribute(member, "name")})
						objects.set_attribute(member, "failed_heist",  0)
					

	elif id=="bonus_malus_timer":
		import bonus
		bonus.bonus_malus_timer()
	elif id=="resting":
		import rest
		rest.rest_hourly_callback()		
	elif id=="cyclic_fatigue_check":
		import rest
		if not hasattr(data("cheat"), "no_fatigue") or not data("cheat").no_fatigue:
			rest.cyclic_fatigue_check()		
		system.notify_global_timer("cyclic_fatigue_check", rest.cyclic_fatigue_check_interval)
	elif id=="fatigue_unconscious_timer":
		if system.get_mission_time()-data("rest").unconscious_start >= timerepr.time("12:0:0"):
			import rest
			for char in system.get_pcs(include_unconscious = True):
				objects.set_attribute(char, "unconscious_subtract_hp", None)
				objects.set_attribute(char, "hitpoints", objects.get_attribute(char, "maxhitpoints"))
				object_events.set_unconscious(char, False)
				rest.rest_heal(12, [])
		else: system.notify_global_timer("fatigue_unconscious_timer", 30*1000)
	elif id=="psychology_check":		# obsolete 19.01.2006; only for old savegames
		pass
	elif string.find( id, "unconscious_timer_" ) == 0:
		character_id = string.replace( id, "unconscious_timer_", "" ) # extract the char id from the timer id
		if character_id in system.get_pcs(include_unconscious = True):
			# test if a char is unconscious. 
			if	objects.is_unconscious( character_id )\
			and (not objects.has_attribute(character_id, "unconscious_subtract_hp") \
				or objects.get_attribute(character_id, "unconscious_subtract_hp") \
				)\
			and objects.is_active(character_id):
					objects.damage( character_id, 1 )
					system.notify_global_timer( id, 60000, False )
	elif string.find( id, "survive_active_timer_" ) == 0:		# obsolete 03.02.2006; only for old savegames
		pass
	# mercenary comment timer
	elif string.find( id, "mercenary_comment_timer" ) != -1:
		# extract mercenary id and event from the timer id
		mercenary_id, event = string.split( id, "_mercenary_comment_timer_" )
		
		if hasattr( data("comments"), "spoken_comments" ):
			spoken_comments = data("comments").spoken_comments
			# check if mercenary has spoken comments
			if spoken_comments.has_key( mercenary_id ):
				mercenary_comments = spoken_comments[ mercenary_id ]

				if mercenary_comments.has_key( event ):
					# Set event to True , that determines that 
					# comments of this type can be made again
					# from now on
					mercenary_comments[ event ][0] = True

	elif string.find(id, "timed_callback_") != -1:
		dummy, callback_id = string.split(id, "timed_callback_")
		callback = data("timed_callback").callbacks.pop(callback_id)
		callback()
	elif id == "mercenary_party_comment_timer":
		comments.make_party_comment()
	elif id == "finish_rest_fallback_timer": # obsolete 06.03.2006
		import rest
		while data("rest").finish_rest_fallback:
			print "############# finish_rest_fallback ##############"
			rest.finish_rest_char_ready(None)
	elif string.find( id, "finish_rest_fallback_timer_" ) == 0:
		if hasattr(data("rest"), "finish_rest_fallback_id") and data("rest").finish_rest_fallback_id == id:
			import rest
			while data("rest").finish_rest_fallback:
				print "############# finish_rest_fallback ##############"
				rest.finish_rest_char_ready(None)
	elif id == "rest_raid_min_time_timer":
		print "rest_raid_min_time_timer"
		global_defines.data("rest").can_continue_rest = True
		continue_rest()
	elif id == "alex_debug":
		#system.add_text(system.get_mission_time())
		import time
		system.add_text("%s - %.2f" % (str(system.get_mission_time()), time.clock()%60))
		system.notify_global_timer('alex_debug', 1000)
	elif id == "rest_raid_seeker_ki_timer":
		if hasattr(data("rest"), "raid") and data("rest").raid:
			if hasattr(data("rest"), "rest_raiders") and len(data("rest").rest_raiders):
				for raider in data("rest").rest_raiders:
					if objects.exists(raider) and not objects.is_destroyed(raider):
						if objects.has_attribute(raider, "foes"):
							if objects.get_distance_to_position(raider, data("rest").rest_x, data("rest").rest_y) > 30:
								callback = "objects.set_attribute('%s', 'foes', ['UISPIELER']);" % raider
								callback += "import object_events;"
								callback += "object_events.on_alert('%s');" % raider
								objects.set_attribute(raider, "foes", None)
								objects.cancel_commandos(raider)
								objects.move_to(raider, data("rest").rest_x, data("rest").rest_y, callback=Lambda(callback))
			system.notify_global_timer("rest_raid_seeker_ki_timer", 1*1000, freezable=False)
	elif string.find( id, "increase_moral_active_timer_" ) == 0: # obsolete 20.01.2006; only for old savegames
		pass
	elif id == 'timer_spoken_party_dialog_en':
		if hasattr( data("comments"), "spoken_party_dialog_en" ): 
			if 'steal' in data("comments").spoken_party_dialog_en.keys():
				if objects.get_distance('MERCENARY_SALLY', data("comments").spoken_party_dialog_en['steal']) > 5:
					comments.make_party_comment()
				else:
					del data("comments").spoken_party_dialog_en['steal']
		

	try:
		if id in data().enemy_script_activity:
			# print '[system_events data().enemy_script_activity: '
			for exp in data().enemy_script_activity[id]:
				exp()
			del data().enemy_script_activity[id]
			data().enemy_script_timer.append(int(id))
	except:
		pass
		#print '[debug system_events_on_time ERROR]: die Variable data().enemy_script_activity oder data().enemy_script_timer ist nict vorhanden'

def on_set_mission_time(old_time, new_time):
	"""Called when a new mission time is set via system.set_mission_time.

Parameters:
        old_time - Old mission time.
        new_time - New mission time.
"""
	pass
	
def on_daytime_change(new_daytime):
	"""Called when the daytime changes, i.e., when the sky switches to a new setting.

Parameters:
        new_daytime - Actual daytime that triggered the sky switch.
"""
	global_defines.data('sky').daytime = new_daytime
	if objects.exists("ALTER_EGO"):
		# only execute if we already have a player party to prevent exceptions
		calc_perception_values(daytime_change = True)

def calc_perception_values(daytime_change = False):
	"""Initializes the perception values for all chars by calling
object_events.calc_perception_values.
"""
	for char in system.get_pcs():
		object_events.calc_perception_values(char, daytime_change)
	for char in system.get_npcs():
		object_events.calc_perception_values(char, daytime_change)
	
def set_follower_char_states():
	"""Sets the follow attribute of following NPCs (chars that have zone_transfer 
attribute for global or this zone).

Called by:
        on_map_loaded()
        on_map_reentry()
        on_post_load().
"""
	location = system.get_act_mission_name()
	
	for char in ['FRANKLIN',								# Arzt aus Z_3
				 'ABEL',									# aus Z_8
				 'ANIE',									# aus Z_7
				 'IAN', 'LINDA', 'PAULA',					# Wohnungssuchende aus Z_6
				 'TIMETRAVELLER',							# Zeitreisender aus Z_2
				 'ROWLAND', 'BANKS',						# Wissenschaftler aus zz_8
				 'REPORTER']:								# Reporter aus Z_2
		if char in system.get_npcs():
			print 'debug [set_follower_char_states: %s]' %char
			if char == 'IAN':
				if objects.get_attribute(char,"face") != "ian_chase":
					objects.set_attribute(char, 'foes', None)
					character.set_approachable_state(char, global_defines.APPROACHABLE)
					objects.set_immortal_state(char, True)
				else:
					pass
			else:
				objects.set_attribute(char, 'foes', None)
				character.set_approachable_state(char, global_defines.APPROACHABLE)
				objects.set_immortal_state(char, True)
	
	for char in ['ABRAHAM', 'MICKEY', 'SUE_Z7', 'FJODR', 	# Garden of Eden Gang Z_6
				 'PRESIDENT_HAYES',									# aus Z_1
				 'WESLEY']:									# aus Z_1
		if char in system.get_npcs():
			print 'debug [set_follower_char_states: %s]' %char
			
			if char == 'PRESIDENT_HAYES':
				if global_defines.was_zone_entered('zone_10.zone_10'):
					if not data('zone_10.zone_10').Hayes_Cutscene_Gelaufen:
						if hasattr(data('zone_10.zone_10'), 'Hayes_flieht'):
							if not data().Hayes_flieht:
								objects.set_attribute(char, 'foes', ['UISPIELER'])
								character.set_approachable_state(char, global_defines.NOTAPPROACHABLE)
								objects.set_immortal_state(char, False)
							else:
								objects.set_attribute(char, 'foes', None)
								character.set_approachable_state(char, global_defines.APPROACHABLE)
								objects.set_immortal_state(char, True)
						else:
							pass
					else:
						objects.set_attribute(char, 'foes', None)
						character.set_approachable_state(char, global_defines.APPROACHABLE)
						objects.set_immortal_state(char, True)
						
			if char == 'WESLEY':
				if location == 'zone_5.zone_5':
					objects.set_attribute(char, 'foes', None)
					character.set_approachable_state(char, global_defines.NOTAPPROACHABLE)
					objects.set_immortal_state(char, True)
				if hasattr(data('zone_1.zone_1'), 'wesley_hat_gestanden'):
					if data('zone_1.zone_1').wesley_hat_gestanden:
						if location == 'zone_1.zone_1':
							if hasattr(data('zone_1.zone_1'), 'WesleyHatVerkackt'):
								if not data('zone_1.zone_1').WesleyHatVerkackt:
									objects.set_attribute(char, 'foes', ['UISPIELER'])
									character.set_approachable_state(char, global_defines.NOTAPPROACHABLE)
									objects.set_immortal_state(char, False)
								else:
									objects.set_attribute(char, 'foes', None)
									character.set_approachable_state(char, global_defines.APPROACHABLE)
									objects.set_immortal_state(char, True)
						else:
							objects.set_attribute(char, 'foes', None)
							character.set_approachable_state(char, global_defines.APPROACHABLE)
							objects.set_immortal_state(char, True)
				else:
					objects.set_attribute(char, 'foes', None)
					character.set_approachable_state(char, global_defines.APPROACHABLE)
					objects.set_immortal_state(char, True)
			if char in ['ABRAHAM', 'MICKEY', 'SUE_Z7', 'FJODR']:
				pass

def handle_vehicles_in_cutscene_on_begin():
	"""Helper function for on_begin_cutscene.
"""
	data().cutszene_vehicles = []
	data().cutszene_chars = []
	for _id in system.get_vehicles():
		import vehicle
		crew = vehicle.get_crew(_id)
		pc_crew_in_vehicle = False
		
		for crew_member in crew:
			if crew_member in system.get_pcs():
				#print "vehicle pos"
				#print objects.get_x(id)
				#print objects.get_y(id)
				vehicle.stop_vehicle(_id,False)
				character.set_out(crew_member)
				if objects.has_attribute(crew_member, 'last_primary_weapon'):
					last_weapon = objects.get_attribute(crew_member, 'last_primary_weapon')
					if last_weapon:
						if last_weapon in objects.get_inventory_id_list(crew_member):
							character.equip_direct(crew_member, last_weapon)
					objects.set_attribute(crew_member, 'last_primary_weapon', '')
				pc_crew_in_vehicle = True
				data().cutszene_chars.append(crew_member )
				
		if (pc_crew_in_vehicle):
			objects.set_active_state(_id,0)
			data().cutszene_vehicles.append(_id)
				
				#data().cutszene_chars.append(crew_member)
				
				#if first_crew_member:
					#location = []
					#location.append(objects.get_x(crew_member ))
					#location.append(objects.get_y(crew_member ))
					
					#data().cutszene_vehicles[id] = location
					#first_crew_member = False
					#objects.set_active_state(id,0)
		
	
def handle_vehicles_in_cutscenes_on_end():
	"""Helper function for on_end_cutscene.
"""
	if hasattr(data(),"cutszene_vehicles"):
		for vehicle_id in data().cutszene_vehicles:
			objects.set_active_state(vehicle_id,1)
			#location = []
			#location = data().cutszene_vehicles[vehicle_id]
			#objects.set_position(vehicle_id,location[0],location[1])
	if hasattr(data(),"cutszene_chars"):
		for cs_char in data().cutszene_chars:
			x_pos= objects.get_x(cs_char)
			y_pos= objects.get_y(cs_char)
			objects.set_position(cs_char,x_pos,y_pos)
	
		if len(system.get_selected_pcs())==0 and len(data().cutszene_chars)>0:
			if "ALTER_EGO" in data().cutszene_chars:
				if objects.exists("ALTER_EGO"):
					system.set_selected_pcs(["ALTER_EGO"])
			else:
				if objects.exists(data().cutszene_chars[0]):
					system.set_selected_pcs([data().cutszene_chars[0]])
	
		
			
def on_begin_fade():
	"""Called whenever system.fade is called (both FADE_IN and FADE_OUT).
"""
	print "on_begin_fade"
	for _id in system.get_vehicles():
		import vehicle
		crew = vehicle.get_crew(_id)
		first_crew_member = True
		
		for crew_member in crew:
			if crew_member in system.get_pcs():
				if first_crew_member:
					vehicle.stop_vehicle(_id,False)
					first_crew_member = False

def rest_finished():
	"""Called when the party has finished resting.
"""
	system.unfreeze_timer()
	
	if system.get_act_mission_name() in ['zone_4.zone_4','zone_3.zone_3']:
		if hasattr(system.get_act_mission(), "rest_finished") and callable(system.get_act_mission().rest_finished):
			system.get_act_mission().rest_finished()
	
def rest_started():
	"""Called when the party starts resting.
"""
	system.freeze_timer()
	if system.get_act_mission_name() in ['zone_4.zone_4','zone_3.zone_3']:
		if hasattr(system.get_act_mission(), "rest_started") and callable(system.get_act_mission().rest_started):
			system.get_act_mission().rest_started()
	
	
	
def rest_raid_started():
	"""Called when the rest gets disrupted by enemies.
"""
	# collect data about throw weapons
	# throw-weapons on the ground are considered loot and teleported after rest
	data("rest").additional_loot = []
	for char in system.get_pcs():
		for item in objects.get_inventory_id_list(char):
			if objects.has_attribute(item, "weapon_type"):
				if objects.get_attribute(item, "weapon_type") == "wurf":
					data("rest").additional_loot.append(item)
	
def rest_raid_finished():
	"""Called when the rest disruption is over.
"""
	pass
	
def rest_canceled():
	"""Called when the rest has been canceled.
"""
	system.unfreeze_timer()
	
	if system.get_act_mission_name() in ['zone_4.zone_4','zone_3.zone_3']:
		if hasattr(system.get_act_mission(), "rest_canceled") and callable(system.get_act_mission().rest_canceled):
			system.get_act_mission().rest_canceled()

# Derff's code is here
def Teleport_To_Car() :
	"""Teleport the whole party to the car"""
	if hasattr(data("ALTER_EGO"), "Last_Vehicle") and can_leave_zone_context_menu():
		vehicle = data("ALTER_EGO").Last_Vehicle
		if system.get_vehicles().count(vehicle) :
			position_x = objects.get_x(vehicle)
			position_y = objects.get_y(vehicle)
			chars = []
			Max_Driving = 0
			Driver = ""
			for pc in system.get_pcs():
				if character.get_vehicle_from_char(pc) == None :
					if objects.has_attribute( pc, "driving") and int(objects.get_attribute( pc, "driving")) > Max_Driving :
						Max_Driving = int(objects.get_attribute( pc, "driving"))
						Driver = pc
					chars.append(pc)
			
			if len(chars) > 0 :
				objects.cancel_commandos(Driver)
				if objects.get_distance(Driver, vehicle) > 10 :
					objects.set_position(Driver, position_x - 3, position_y - 3  )
				character.get_in(Driver, vehicle)
				chars.remove(Driver)
			
			i = 1
			for pc in chars	:
				objects.cancel_commandos(pc)
				if objects.get_distance(pc, vehicle) > 15 :				
					objects.set_position(pc, position_x - 3-i, position_y - 3-i  )
				character.get_in(pc, vehicle)
				i += 1

import sys
sys.modules["system"].Teleport_To_Car = Teleport_To_Car
